@@ -91,7 +91,11 @@ pub struct Release {
 }
 
 fn deserialize_binaries(value: &str) -> Result<Vec<String>, String> {
-    Ok(value.split_whitespace().map(|s| s.to_string()).collect())
+    Ok(value.split(",").map(|s| s.trim().to_string()).collect())
+}
+
+fn deserialize_testsuite_triggers(value: &str) -> Result<Vec<String>, String> {
+    Ok(value.split(",").map(|s| s.trim().to_string()).collect())
 }
 
 fn join_lines(components: &[String]) -> String {
@@ -131,12 +135,16 @@ pub struct Source {
 
     #[deb822(field = "Build-Depends")]
     /// Build dependencies of the source
-    pub build_depends: Option<String>,
+    pub build_depends: Option<Relations>,
 
     #[deb822(field = "Build-Depends-Indep")]
     /// Build dependencies independent of the architecture of the source
     pub build_depends_indep: Option<Relations>,
 
+    #[deb822(field = "Build-Depends-Arch")]
+    /// Build dependencies dependent on the architecture of the source
+    pub build_depends_arch: Option<Relations>,
+
     #[deb822(field = "Build-Conflicts")]
     /// Build conflicts of the source
     pub build_conflicts: Option<Relations>,
@@ -145,6 +153,10 @@ pub struct Source {
     /// Build conflicts independent of the architecture of the source
     pub build_conflicts_indep: Option<Relations>,
 
+    #[deb822(field = "Build-Conflicts-Arch")]
+    /// Build conflicts dependent on the architecture of the source
+    pub build_conflicts_arch: Option<Relations>,
+
     #[deb822(field = "Standards-Version")]
     /// Standards version of the source
     pub standards_version: Option<String>,
@@ -161,6 +173,10 @@ pub struct Source {
     /// Testsuite of the source
     pub testsuite: Option<String>,
 
+    #[deb822(field = "Testsuite-Triggers", deserialize_with = deserialize_testsuite_triggers, serialize_with = join_whitespace)]
+    /// The packages triggering the testsuite of the source
+    pub testsuite_triggers: Option<Vec<String>>,
+
     #[deb822(field = "Vcs-Browser")]
     /// VCS browser of the source
     pub vcs_browser: Option<String>,
@@ -197,6 +213,10 @@ pub struct Source {
     /// VCS Mtn of the source
     pub vcs_mtn: Option<String>,
 
+    #[deb822(field = "Dgit")]
+    /// Dgit information (commit, suite, ref, url)
+    pub dgit: Option<crate::fields::DgitInfo>,
+
     #[deb822(field = "Priority")]
     /// Priority of the source
     pub priority: Option<crate::fields::Priority>,
@@ -483,4 +503,121 @@ Suggests: apt-doc, aptitude | synaptic | wajig
             })
         );
     }
+
+    #[test]
+    fn test_source() {
+        let source = r#"Package: abinit
+Binary: abinit, abinit-doc, abinit-data
+Version: 9.10.4-3
+Maintainer: Debichem Team <debichem-devel@lists.alioth.debian.org>
+Uploaders: Andreas Tille <tille@debian.org>, Michael Banck <mbanck@debian.org>
+Build-Depends: debhelper (>= 11), gfortran, liblapack-dev, python3, graphviz, markdown, ghostscript, help2man, libfftw3-dev, libhdf5-dev, libnetcdff-dev, libssl-dev, libxc-dev, mpi-default-dev, python3-dev, python3-numpy, python3-pandas, python3-yaml, texlive-latex-extra, texlive-fonts-recommended, texlive-extra-utils, texlive-pstricks, texlive-publishers, texlive-luatex
+Architecture: any all
+Standards-Version: 3.9.8
+Format: 3.0 (quilt)
+Files:
+ 843550cbd14395c0b9408158a91a239c 2464 abinit_9.10.4-3.dsc
+ a323f11fbd4a7d0f461d99c931903b5c 130747285 abinit_9.10.4.orig.tar.gz
+ 27c12d3dac5cd105cebaa2af4247e807 15068 abinit_9.10.4-3.debian.tar.xz
+Vcs-Browser: https://salsa.debian.org/debichem-team/abinit
+Vcs-Git: https://salsa.debian.org/debichem-team/abinit.git
+Checksums-Sha256:
+ c3c217b14bc5705a1d8930a2e7fcef58e64beaa22abc213e2eacc7d5537ef840 2464 abinit_9.10.4-3.dsc
+ 6bf3c276c333956f722761f189f2b4324e150c8a50470ecb72ee07cc1c457d48 130747285 abinit_9.10.4.orig.tar.gz
+ 80c4fb7575d67f3167d7c34fd59477baf839810d0b863e19f1dd9fea1bc0b3b5 15068 abinit_9.10.4-3.debian.tar.xz
+Homepage: http://www.abinit.org/
+Package-List: 
+ abinit deb science optional arch=any
+ abinit-data deb science optional arch=all
+ abinit-doc deb doc optional arch=all
+Testsuite: autopkgtest
+Testsuite-Triggers: python3, python3-numpy, python3-pandas, python3-yaml
+Directory: pool/main/a/abinit
+Priority: optional
+Section: science
+"#;
+
+        let source: Source = source.parse().unwrap();
+
+        assert_eq!(source.package, "abinit");
+        assert_eq!(source.version, "9.10.4-3".parse().unwrap());
+        assert_eq!(
+            source.binaries,
+            Some(vec![
+                "abinit".to_string(),
+                "abinit-doc".to_string(),
+                "abinit-data".to_string()
+            ])
+        );
+
+        let build_depends = source.build_depends.as_ref();
+        let build_depends: Vec<_> = build_depends.iter().collect();
+        let build_depends = build_depends[0];
+
+        let expected_build_depends = &[
+            "debhelper",
+            "gfortran",
+            "liblapack-dev",
+            "python3",
+            "graphviz",
+            "markdown",
+            "ghostscript",
+            "help2man",
+            "libfftw3-dev",
+            "libhdf5-dev",
+            "libnetcdff-dev",
+            "libssl-dev",
+            "libxc-dev",
+            "mpi-default-dev",
+            "python3-dev",
+            "python3-numpy",
+            "python3-pandas",
+            "python3-yaml",
+            "texlive-latex-extra",
+            "texlive-fonts-recommended",
+            "texlive-extra-utils",
+            "texlive-pstricks",
+            "texlive-publishers",
+            "texlive-luatex",
+        ];
+
+        assert_eq!(build_depends.len(), expected_build_depends.len());
+        assert_eq!(build_depends[0][0].name, expected_build_depends[0]);
+        assert_eq!(
+            build_depends[build_depends.len() - 1][0].name,
+            expected_build_depends[build_depends.len() - 1]
+        );
+
+        assert_eq!(
+            source.testsuite_triggers,
+            Some(
+                ["python3", "python3-numpy", "python3-pandas", "python3-yaml"]
+                    .into_iter()
+                    .map(String::from)
+                    .collect()
+            )
+        );
+    }
+
+    #[test]
+    fn test_source_with_dgit() {
+        let source = r#"Package: test-pkg
+Version: 1.0-1
+Maintainer: Test Maintainer <test@example.com>
+Dgit: c1370424e2404d3c22bd09c828d4b28d81d897ad debian archive/debian/1.1.0 https://git.dgit.debian.org/test-pkg
+Directory: pool/main/t/test-pkg
+Package-List: 
+ abinit deb science optional arch=any
+ abinit-data deb science optional arch=all
+ abinit-doc deb doc optional arch=all
+"#;
+
+        let source: Source = source.parse().unwrap();
+        assert_eq!(source.package, "test-pkg");
+        let dgit = source.dgit.as_ref().unwrap();
+        assert_eq!(dgit.commit, "c1370424e2404d3c22bd09c828d4b28d81d897ad");
+        assert_eq!(dgit.suite, "debian");
+        assert_eq!(dgit.git_ref, "archive/debian/1.1.0");
+        assert_eq!(dgit.url, "https://git.dgit.debian.org/test-pkg");
+    }
 }
